package Sub::Spec::To::Text;

our $VERSION = '0.04'; # VERSION

1;
# ABSTRACT: Generate text documentation from spec


__END__
=pod

=head1 NAME

Sub::Spec::To::Text - Generate text documentation from spec

=head1 VERSION

version 0.04

=head1 SYNOPSIS

 # use Sub::Spec::To::Text qw(spec_to_text);
 # my $text = spec_to_text(spec=>$spec, ...);

=head1 DESCRIPTION

EMPTY MODULE AT THE MOMENT.

=head1 SEE ALSO

L<Sub::Spec::To::Text::Usage>

=head1 AUTHOR

Steven Haryanto <stevenharyanto@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2011 by Steven Haryanto.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

